#include <dbsim.h>
#include <object.h>
#include <node.h>
#include <language.h>
#include <db.h>
#include <strings.h>
#include <math.h>

//---------------------------------------------------------------------------
// class implementation for Indexes
//---------------------------------------------------------------------------

Index::Index(int myIndexNo, File* myDataFile, int myKeySize, int myIndexType)
{
    // index initialization variables
    Node* node;
    int n, d, disk;

    // record the incoming file and key information
    indexNo = myIndexNo;
    data = myDataFile;
    keySize = myKeySize;
    indexType = myIndexType;

    // clear out the sparse fragment array
    for (n = 0; n < MAX_NODES; n++) {
        for (d = 0; d < MAX_DISKSPERNODE; d++) {
	    fragment[n][d] = NULL;
	}
    }

    // now allocate a set of index fragments for representing the index,
    // with one index fragment allocated for each file fragment
    for (n = 0; n < data->numNodes; n++) {
        for (d = 0; d < data->numDisks; d++) {
	    node = data->storageNode(n);
	    disk = data->storageDisk(d);
	    fragment[node->id][disk] =
	        new IndexFrag(data->getFrag(node, disk), keySize, indexType);
        }
    }
}


Index::~Index()
{
    for (int n = 0; n < data->numNodes; n++) {
        for (int d = 0; d < data->numDisks; d++) {
	    delete fragment[data->storageNode(n)->id][data->storageDisk(d)];
        }
    }
}


void Index::print()
{
    int n, d;

    printf("Summary of index %d of file %d...\n", indexNo, data->fileNo);
    printf("\tKey size is %d, index fragments are...\n", keySize);
    for (n = 0; n < data->numNodes; n++) {
        for (d = 0; d < data->numDisks; d++) {
  	    printf("\t  Displaying index fragment[%d, %d]\n",
  	            data->storageNode(n)->id, data->storageDisk(d));
  	    fragment[data->storageNode(n)->id][data->storageDisk(d)]->print();
        }
    }
}

