#include "dbsim.h"
#include "node.h"
#include "op.h"
#include "msg.h"
#include "mailbox.h"
#include "split.h"
#include "cost.h"

// constructor definitions for Op nodes and SplitTables

SplitTable::SplitTable(int size, int tupSize)
{
    curOutBuf = 0;
    splitCnt = size;
    tupleSize = tupSize;
    splitArray = new NetBufRec[size];
};
    
SplitTable::~SplitTable()
{
    delete [splitCnt] splitArray;
};

void SplitTable::sendNetBuffer(int bufIdx, bool eofFlag, Node* n, Mailbox* source, int msgType)
{
    register DataMsg 	*dMsg;
    int instCnt;

    dMsg = DataMsg::alloc();  // allocate a free buffer
    dMsg->numTuples = splitArray[bufIdx].numTuples;
    dMsg->tupleSize = tupleSize;
    dMsg->eofFlag = eofFlag;

    if ((bufIdx < 0) || (bufIdx >= splitCnt))
	printf("SendNetBuffer: Illegal bufIdx=%d, splitCnt=%d\n",
		bufIdx, splitCnt);
    
    if (msgType == NORMAL)  {
        // charge node for copying tuples to output page
	instCnt = splitArray[bufIdx].numTuples * 
	    (INST_SPLIT + tupleSize * INST_BCOPY) ;
	
	n->cpu.use(((float) instCnt) / (Mips * 1000)); 
        if (source->owningOp != NULL) 
          source->owningOp->cnt_CPUInstr += instCnt;

	splitArray[bufIdx].numTuples = 0;
	// now send off the message
	dMsg->send(splitArray[bufIdx].destNode, source);
    } 
    else {
        splitArray[bufIdx].numTuples = 0;
	// now send off the message
	dMsg->cheapSend(splitArray[bufIdx].destNode, source);
    }
}

void SplitTable::flushNetBufs(Node* n, Mailbox* source, int msgType)
{
    register int  i;
    for (i=0;i<splitCnt;i++) sendNetBuffer(i, TRUE, n, source, msgType);
}
