#include "dbsim.h"
#include "object.h"
#include "node.h"
#include "msg.h"
#include "mailbox.h"
#include "query.h"
#include "sched.h"
#include "language.h"

///////////////////////////////////////////////////////////////////////////////
//
//    Name:                GSched.c
//
//    Function:        Global Scheduler process
//                                 
//    Change Activity:
//
//         08/xx/91 (KPB) - Initial version.
//
///////////////////////////////////////////////////////////////////////////////




// Global variables to maintain the average MPL of the system
double PrevMPLChange = 0;  // last time a query admitted/departed
int CurrentMPL = 0;


extern Mailbox* GlobalSchedPort;
extern MemoryQueue *GlobalQueryWaitQueue;
extern initialSchedule(QueryMsg*, Schedule*&, Schedule*&);
extern allocateResources(QueryMsg*, Schedule*&, Schedule*&);
extern void QueryScheduler(Node* ,QueryMsg*, Schedule*, Schedule*);
extern CanRun(QueryPlan*);
void DeleteSched(Schedule*&);


void startQuery(Node*, QueryMsg*);
void globalScheduler(Node*); 




void globalScheduler(Node* gSchedNode) 
{
    JobClass *queryClass;
    QueryMsg *reqMsg;                                             
    int		transCounter=0;

    create("GlobalScheduler");  
    if (Flags[TRACE_CONTROL]) 
      printf("Global scheduler process started on node %d\n", gSchedNode->id);


    while (ActiveTerms > 0) 
    {    
        // while there are people to listen to
        // Waits for requests from terminals 

        reqMsg = (QueryMsg*) GlobalSchedPort->recv();

        // Forks off a query scheduler process to hand this specific query.
        // Right now, just runs em all on this node.    Need to load bal this?

	reqMsg->query->transId = transCounter++;

	queryClass = reqMsg->query->jobClass;


        if (Flags[TRACE_SCHED])
	      printf("max %d current %d\n", 
		     queryClass->specifiedMaxMPL, queryClass->currentMPL);

	// if the currentMPL of class already at maximum
        //    queue up the query in the job class queue
	//    queries are dequeued when a query from this class
	//    completes
	if (queryClass->currentMPL == queryClass->specifiedMaxMPL)
	{
            if (Flags[TRACE_SCHED])
	         printf("currentMPL = maxMPL query waits\n");
	    queryClass->queryWaitQueue->enqueue(gSchedNode, reqMsg,
					    NULL, NULL);
        }
	else
	    // start scheduling the query
	    startQuery(gSchedNode, reqMsg);
    }
    delete GlobalQueryWaitQueue;
}





// start scheduling the query
void startQuery(Node* gSchedNode, QueryMsg* reqMsg)
{
    int e = OK;
    Schedule *schedule, *waitSchedule;


    schedule = NULL;
    waitSchedule = NULL;

    e = initialSchedule(reqMsg, schedule, waitSchedule);
    if (e == OK)
    {
        reqMsg->query->jobClass->currentMPL++;

        // if the global query queue is not empty add the query 
	// to the queue
	if ((GlobalQueryWaitQueue->nonempty()) &&
	    (GlobalQueryWaitQueue->checkWaiters(reqMsg->query->totalMaxPagesNeeded)))
	{
	  
	     GlobalQueryWaitQueue->enqueue(gSchedNode, reqMsg, 
					   schedule, waitSchedule);
	     if (Flags[TRACE_SCHED])
	         printf("Query Queue not empty : new query queued up\n");
	}
	else
	    if (CanRun(reqMsg->query))
	    {
	        // form the schedules. also does the resource allocation
		e = allocateResources(reqMsg, schedule, waitSchedule);
		
		if (e == OK)
		    // fork off a new query scheduler to handle the query
		    QueryScheduler(gSchedNode, reqMsg, schedule, waitSchedule);
		else
		    // report scheduling error to the user
		    printf("*ERROR*: in scheduling query. abort execution\n");
		  
	    }
	    else
	    {
	        GlobalQueryWaitQueue->enqueue(gSchedNode, reqMsg, 
					      schedule, waitSchedule);
		if (Flags[TRACE_SCHED])
		    printf("Query cannot run now: queued up\n");
	    }
    }
    else // e == NOTOK
    {
        // destroy the old schedules
	DeleteSched(schedule);
	DeleteSched(waitSchedule);
    }

}




